;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Loading this file installs CG/JS.
;; Please see readme.txt for more information
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(in-package :user)

(eval-when (compile eval load)
  (require :shell)
  (use-package :excl.shell))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; validate files

(let ((platform-file
       (directory
	(merge-pathnames
	 "PLATFORM-*"
	 (path-pathname *load-pathname*)))))
  (or platform-file
      (error "Could not find the PLATFORM file."))
  
  (when (> (length platform-file) 1)
    (error "There is more than one PLATFORM file: ~a." platform-file))
  (setq platform-file (car platform-file))
  
  (or (=~ "^PLATFORM-(.*)" (file-namestring platform-file))
      (error "Cannot parse the PLATFORM file."))
  
  (let ((expected-platform $1)
	(current-platform
	 (format nil "~a~a" (symbol-name sys::.os.) #+smp "smp" #-smp "")))
    (or (string-equal expected-platform current-platform)
	(error
	 "The current platform (~a) does not match the downloaded version (~a)."
	 current-platform expected-platform)))
  ;; We are on the correct platform, fall through...
  )

;; HTML files needed for the operation of CG/JS
;; (this variable's value is generated by build_cgjs.sh)
(defvar *html-files* '("cgjs.html" "ssl-not-found.html" "browser-not-supported.html" ))

(defvar *extra-files*)
(setq *extra-files* nil)

(dolist (file *html-files*)
  (setq file (merge-pathnames file *load-pathname*))
  (or (probe-file file)
      (error "~a does not exist." file))
  (push file *extra-files*))
;; All *html-files* exist...

;; These two are directories but the pathname is specified as
;; a file, so merging works below.
(defvar *cg2js-directory* (merge-pathnames "code.cg2js" *load-pathname*))
(defvar *idejs-directory* (merge-pathnames "code.idejs" *load-pathname*))

(dolist (d (list *cg2js-directory* *idejs-directory*))
  (or (probe-file d)
      (error "~a does not exist." d)))

;; MD5 checksums of the incoming files
;; (this variable's value is generated by build_cgjs.sh)
(defvar *checksums* 
'(
(#x8a816a53d273984f5e798e95a8233a57 . "./code.cg2js/cg-common-control.fasl")
(#xce13e49d41b9fe955e295fa40cc58fad . "./code.cg2js/cg-clipboard-stack.fasl")
(#xd39aabfe53c1a7fe9fd7fba56b15363d . "./code.cg2js/cg-yes-no-string.fasl")
(#x7989e61a77e49409cbec5bc33e7cc989 . "./code.cg2js/cg-clipboard.fasl")
(#x00b6946f7a5f3868ec9c5216f24e05c0 . "./code.cg2js/cg-dde-utils.fasl")
(#xe23f40218c1f9c086722efc5b3eb4ae2 . "./code.cg2js/cg-editable-text.fasl")
(#x72d92ebe746791bd945d1c46a3bc6a76 . "./code.cg2js/cg-player.fasl")
(#xd4e056d803be71e720ab88a45a26c876 . "./code.cg2js/cg-text-or-combo.fasl")
(#x0e5a535f19c6f00c2b2df6d1bb22b862 . "./code.cg2js/cg-picture-widget-palette.fasl")
(#x6b11f2984c7973833876c45d20040528 . "./code.cg2js/cg-graphics-context.fasl")
(#x687d6ac6a05b22b0234e5dfff9d34879 . "./code.cg2js/cg-choose-printer.fasl")
(#x7af05b10e7ce3c0115495b0ea456be1a . "./code.cg2js/cg-scroll-bar-mixin.fasl")
(#x8d9bb4d547ff00af1f74fab170c418fe . "./code.cg2js/cg-edit-in-place.fasl")
(#xf7386912c5052283c0878f7f8e8a250d . "./code.cg2js/cg-split-bar.fasl")
(#x08fc98743de3631a4854896f6226c440 . "./code.cg2js/cg-timer.fasl")
(#x81862ed79a5531a86e041c4aec481a71 . "./code.cg2js/cg-progress-indicator.fasl")
(#x6093fe78c3570068cf27a0b316e9698b . "./code.cg2js/cg-object-editor.fasl")
(#xffca2250c18492b0706285db6a419237 . "./code.cg2js/cg-fill-texture.fasl")
(#x0151dadfe945b19024bc0bf4d8612969 . "./code.cg2js/cg-multi-picture-button-drag-and-drop.fasl")
(#x3004cf7fe7cf4f26d5d416d2c1c35dbe . "./code.cg2js/cg-os-window.fasl")
(#x7a0d82b442a56925853faa5768cca833 . "./code.cg2js/cg-color-dialog.fasl")
(#xf96c3236559a2fbd41c84d55eb32c27c . "./code.cg2js/cg-combo-box.fasl")
(#x47ca20475a02877fcb79e3f10d570dd9 . "./code.cg2js/cg-directory-dialog.fasl")
(#xa3d4f43a6f76e0b1d10ee5cb28b5fc1b . "./code.cg2js/cg-tooltip.fasl")
(#x37d7f795534c8e98f2073b9055922087 . "./code.cg2js/cg-lettered-menu.fasl")
(#x0af10dfb3b029ef1898a91c742d25754 . "./code.cg2js/cg-clipboard-pixmap.fasl")
(#xe4c37deaa6358fc6bcf399c432819674 . "./code.cg2js/cg-pixmap.fasl")
(#xc2c9c841406cefa05cd334b22014fdee . "./code.cg2js/cg.fasl")
(#x3a43e61cff6653691f85b6650403582c . "./code.cg2js/cg-class-slot-grid.fasl")
(#x1bab701f84e08372d77f11767cfa6511 . "./code.cg2js/cg-template-string.fasl")
(#x23d54d13abd52b386163cedce4217dac . "./code.cg2js/cg-text-edit-pane-mark.fasl")
(#x1707a0f7ebc7848c3d1c6846d22b2331 . "./code.cg2js/cg-check-box.fasl")
(#x4f1a49d324929d1de6518a257bac402f . "./code.cg2js/cg-toggling-widget.fasl")
(#x7665e08ad9892951dd29cb6d5db0864c . "./code.cg2js/cg-radio-button.fasl")
(#xcbc1e440feaca40e9f0c37816afd963e . "./code.cg2js/cg-directory-dialog-os.fasl")
(#x8c9b7fda1117f273e96f4401b37f6234 . "./code.cg2js/cg-scaling-stream.fasl")
(#x65030a3a1fcb6445f7607fdfdd0f4b54 . "./code.cg2js/cg-multi-picture-button.fasl")
(#x21d2f9616d4bbc516be4ada2c165b014 . "./code.cg2js/cg-multi-line-editable-text.fasl")
(#x8ffaac44a848919327104208d3cb5436 . "./code.cg2js/cg-grid-widget.fasl")
(#x239ad2309c770ee2113302316a6ed888 . "./code.cg2js/cg-text-edit-pane.fasl")
(#x21984eb91b191af25a40f0695e5c05d4 . "./code.cg2js/cg-item-list.fasl")
(#x3d449b2dd5e0f0c48d97e25f5015a1fc . "./code.cg2js/cg-paren-matching.fasl")
(#x94638534070b9f05a6dd1c0b1a444d0a . "./code.cg2js/cg-multi-picture-button-tooltip.fasl")
(#x2767fccae13e21627ce97c640d6541e6 . "./code.cg2js/cg-html-dialog.fasl")
(#x61b00875e7b88c228476cb0465c3ed2a . "./code.cg2js/cg-pixmap-widget.fasl")
(#x73b8297a331074d97ac423c6e11d5075 . "./code.cg2js/cg-project-window.fasl")
(#xb8f397223af62f82f1ba2d3eced5251e . "./code.cg2js/cg-toolbar.fasl")
(#xac3af836bbfbd4f81ade90a31be9f498 . "./code.cg2js/cg-caret.fasl")
(#xad54625e87b5b221e4a37beac1e27381 . "./code.cg2js/cg-scroll-bar.fasl")
(#x4df3275c2345e2c9c7f60865903bab70 . "./code.cg2js/cg-web-browser.fasl")
(#x003ffee2209f06134ebbce2af6f8bb93 . "./code.cg2js/cg-dropping-outline.fasl")
(#x8478640454b3c6484dec4791a13c275f . "./code.cg2js/cg-font-dialog.fasl")
(#xf4b49dc70f35d88fcdc54792022ef169 . "./code.cg2js/cg-icon-pixmap.fasl")
(#x8b783c7fe9dd06c97d63fa4047540697 . "./code.cg2js/cg-yes-no-list.fasl")
(#xc651ae201005539fa869f66bb74e49a4 . "./code.cg2js/cg-lisp-edit-pane.fasl")
(#x71918416364b734dc41dec639ccfd399 . "./code.cg2js/cg-lamp.fasl")
(#xc203e51456013892045770e719ad0413 . "./code.cg2js/cg-drawable.fasl")
(#x67db5701383fd8dff6d2c98a7462f89d . "./code.cg2js/cg-choice-list.fasl")
(#x001386b1501ca8f5f9e64a91b2f36a82 . "./code.cg2js/cg-curve.fasl")
(#xc6715e3aa73f1e9994408ce7bd4f2239 . "./code.cg2js/cg-base.fasl")
(#x0b1157c271edc6d6878d6cad2c32ed79 . "./code.cg2js/cg-keyboard-shortcuts.fasl")
(#x32d49a4fd14e7a5c727fd844fec1ab02 . "./code.cg2js/cg-printing.fasl")
(#x53b955e6bceb70bbd8fea32a05d78cd0 . "./code.cg2js/cg-chart-or-plot.fasl")
(#x66f769d572373e416a51ef9d61bff214 . "./code.cg2js/cg-hotspot.fasl")
(#x40c32ab27b60464808f811217631d68c . "./code.cg2js/cg-object-editor-layout.fasl")
(#x2674d2d853f5de8f48a644173e6c7130 . "./code.cg2js/cg-utility-dialog.fasl")
(#x29d102aed5d810351472e8ac6ceb3ea3 . "./code.cg2js/cg-shortcut-menu.fasl")
(#x26023b71f6b43837748190f0cd9c3f1e . "./code.cg2js/cg-comtab.fasl")
(#xfc73c85befd42535efc9615571d3ee99 . "./code.cg2js/cg-os-widget.fasl")
(#x4cce56d6c00f8a4b21371199e18c5281 . "./code.cg2js/cg-icon.fasl")
(#x85c861d6b16274c47104991b0d556544 . "./code.cg2js/cg-dev.fasl")
(#x4797afddf89221e71a4dc6d5e5242b1a . "./code.cg2js/cg-calendar.fasl")
(#xc44aee07908b41bc6d511e47556a61ab . "./code.cg2js/cg-group-box.fasl")
(#xc50c26fe6e1d4add87733a1b2264e782 . "./code.cg2js/cg-grid-widget-drag-and-drop.fasl")
(#xf16e111342b57da5d9e8f14fb98d38de . "./code.cg2js/cg-bitmap-pane.fasl")
(#x285071987d31cb177350cad8a10b2cbf . "./code.cg2js/cg-drag-and-drop.fasl")
(#x91bb4b3356a79b8c480cb355e346207b . "./code.cg2js/cg-nodes.fasl")
(#x3d3ee6f2b9c0c993f5b483ef5866da4e . "./code.cg2js/cg-intersect.fasl")
(#xba9d3b0609ed937490f9c0ff6471865d . "./code.cg2js/cg-outline-drag-and-drop.fasl")
(#x3802d20bc849cd032b1bd4667a8f4765 . "./code.cg2js/cg-class-support.fasl")
(#xeaac0399a648b11f17d170237de4f956 . "./code.cg2js/cg-dialog-item.fasl")
(#xe6228a1f407a42341a3086f74ac6210b . "./code.cg2js/cg-plot-widget.fasl")
(#x686c58dfe314428de209f1242c48482c . "./code.cg2js/cg-pixmap-printing.fasl")
(#xaf1f16fc5c54bb574ffe0b2c03e6cb74 . "./code.cg2js/cg-up-down-control.fasl")
(#xd4c57a3aa3b186d1441d7343945c64b0 . "./code.cg2js/cg-drag-and-drop-image.fasl")
(#xd9d0958c7cc64f5d1d70ebf34a6fed6a . "./code.cg2js/cg-status-bar.fasl")
(#xa0b3514d444455f1bd49676cde9c966c . "./code.cg2js/cg-class-grid.fasl")
(#x5e7e483ce868cd9602e8cc1d22993e7c . "./code.cg2js/cg-wrap-string.fasl")
(#xa65242898506e70b8fb7404ad4393ad2 . "./code.cg2js/cg-layout.fasl")
(#x50bed8fc38d0bff8ec22fb6b43034676 . "./code.cg2js/cg-lisp-widget.fasl")
(#x35914bd648ec7972356fe8a561d8f621 . "./code.cg2js/cg-button.fasl")
(#x44e936526a89e9e38c0f3a3edc3eae88 . "./code.cg2js/cg-string-dialog.fasl")
(#xb90e52c0c20f61f0c64374bd0c08e128 . "./code.cg2js/cg-pixmap-rotate.fasl")
(#x6851ec3fdec1350c563f62a55df432d6 . "./code.cg2js/cg-outline-edit-in-place.fasl")
(#x0e3ac7e4d35e8927564ff7ccbd256d91 . "./code.cg2js/cg-bitmap-stream.fasl")
(#x96ba2bc8bfdb6465b82a0aa3aa418a59 . "./code.cg2js/cg-static-text.fasl")
(#x3e87a070c0f2dffd95fa449aabca4aa3 . "./code.cg2js/cg-outline.fasl")
(#x1ea9f12ba8dd0b729cd5fd4ebc2abfb1 . "./code.cg2js/cg-gesture-emulation.fasl")
(#x8032ef5f942c486e2b3e93541b27410e . "./code.cg2js/cg-bitmap-pane-clipboard.fasl")
(#xfd10b3aff7ad2e45ba9f9d17e43b35ff . "./code.cg2js/cg-picture-widget.fasl")
(#xe8a1106b50f7d785ff0261e14ba080d8 . "./code.cg2js/cg-multi-line-lisp-text.fasl")
(#xead2270ab8e50df2b62b556cde7a1d67 . "./code.cg2js/cg-chart-widget.fasl")
(#xceb1285fa20a3361070ee863ddf172a1 . "./code.cg2js/cg-cursor-pixmap.fasl")
(#x60c6bbee2006eed3f92a285ac28c0e9e . "./code.cg2js/cg-lisp-text.fasl")
(#x54a0083793c65172e6778eabd35dc4dd . "./code.cg2js/cg-palette.fasl")
(#x55ba322906fe90f1192465fff9490465 . "./code.cg2js/cg-property.fasl")
(#xb85ef82ffba4cc6b0db9764f529f9678 . "./code.cg2js/cg-intersect-posbox.fasl")
(#xbe43b623d6dd65f3de76db85e5ab4c93 . "./code.cg2js/cg-file-dialog.fasl")
(#x1a0422728c3e8d45a56d044376ba2c23 . "./code.cg2js/cg-text-edit-pane-file-io.fasl")
(#xffd5295b160e4a9985048f7f2af59788 . "./code.cg2js/cg-get-pixmap.fasl")
(#x74e74dc2ee0eb57ca3a116c490c53aec . "./code.cg2js/cg-tray.fasl")
(#xd27fae08e5884bd51ab0716966089ccf . "./code.cg2js/cg-acache.fasl")
(#x5ba874def69cbb0c4a392c00120017a0 . "./code.cg2js/cg-message-dialog.fasl")
(#x7419c11c7962726e9ccf46ec2d93b10d . "./code.cg2js/cg-pixmap-file-io.fasl")
(#x08aa4d2fef66aaa1bcf845ebefbd25ff . "./code.cg2js/cg-drawable-clipboard.fasl")
(#x1c0cb232685476ce14a9613a85491c9a . "./code.cg2js/cg-tab-control.fasl")
(#x1f902addc7e265b002d310f58deee2cb . "./code.cg2js/cg-scrolling-static-text.fasl")
(#x4c4e52d47de4e31739d10c41535818b5 . "./code.cg2js/cg-menu-tooltip.fasl")
(#x853add740464499bb64d76287294f634 . "./code.cg2js/cg-find-string-dialog.fasl")
(#x50406fc7cc68e10b8315c907ac10083e . "./code.cg2js/cg-menu.fasl")
(#x442c4b8eec39ac2f4a33f69636f6ee46 . "./code.cg2js/cg-get-position.fasl")
(#x2ce75361fecc7976983ad50a390d382b . "./code.cg2js/cg-trackbar.fasl")
(#x94c8638ee98e73619b2eff9186243285 . "./code.cg2js/cg-selected-object.fasl")
(#x41896a96a9a23f6fd9445304018d17ec . "./code.cg2js/cg-list-view.fasl")
(#x58c91b7abc025a03557a4691a6898a37 . "./code.cg2js/cg-html-widget.fasl")
(#x4569f11abc5176999354a56ac93e5d19 . "./code.cg2js/cg-sample-file-menu.fasl")
(#xee318054f4a43a1f070973ab6892b581 . "./code.cg2js/cg-text-widget.fasl")
(#x1425fd74910297c6f5e0f94aab816aaa . "./code.idejs/ide-base.fasl")
(#x710df125ff9b3813b89bf76802e80429 . "./code.idejs/ide-process-browser.fasl")
(#x088b9a775309f84bc06eb55324ea124f . "./code.idejs/ide-patch-dialog.fasl")
(#x585c2aab10e65eb6e69d67fc5f6ee55d . "./code.idejs/ide-keyshort.fasl")
(#x6a49a08932291919b78ee6b75f65ee60 . "./code.idejs/ide-window-list.fasl")
(#xa2dec62fb2401834ec0a11e7249a925d . "./code.idejs/ide.fasl")
(#x9808bd40a381acf235875b27adb1424c . "./code.idejs/ide-editor.fasl")
(#x2e3359334de676ffd8db2db204ffe96b . "./code.idejs/ide-clipboard.fasl")
(#x3e7964b2353436f84a3631bf4aec0611 . "./code.idejs/ide-trace.fasl")
(#xaf90ba02b3113085e9c473a2c0cdbb1f . "./code.idejs/ide-class-browser.fasl")
(#xb8eb7ba229b4327659d832e238599d2c . "./code.idejs/ide-grapher.fasl")
(#xf0560a9857e48cdd8d4ac3c6c87c8db0 . "./code.idejs/ide-apropos.fasl")
(#x529dcbeb5cd03f6d3954b5f0f4369999 . "./code.idejs/ide-navigator.fasl")
(#xbed9d2b3f333e1e14d86fa1305f674a3 . "./code.idejs/ide-find-definitions.fasl")
(#xd086fd997bdfea50394a0492b6629da3 . "./code.idejs/ide-project.fasl")
(#x04a1b52edf9a40015b2bcfddf17ac3fa . "./code.idejs/ide-class-grapher.fasl")
(#xe15fb207f8becbfe60a87761e54a3a89 . "./code.idejs/ide-acache.fasl")
(#xbe3e21c2c1330dc5f27a46fa42516223 . "./code.idejs/ide-find-in-files.fasl")
(#x5b09e43deabf6c799a88e09079af71b2 . "./code.idejs/ide-inspector.fasl")
(#xa00144af635af69543c2658cbd65dda6 . "./code.idejs/ide-intro.fasl")
(#xa2a7a955dc0944d24d295e929e10c7b5 . "./code.idejs/ide-package-list.fasl")
(#x43ec4d8f964ec04e763416e337095e95 . "./code.idejs/ide-dev.fasl")
(#x62fc7253323d4ceae3a8cabba44ab9de . "./code.idejs/ide-form.fasl")
(#x9a8f50ffdba20a10156ef75920aa924b . "./cgjs.html")
(#xdac1739373835d1a63cbafa2c3e0b06a . "./ssl-not-found.html")
(#x90c6d2a65504d45266ad9fa3a49be2f9 . "./browser-not-supported.html"))
 )

(let ((files-ok
       (dolist (checksum *checksums* t)
	 (when (/= (car checksum)
		   (md5-file
		    (merge-pathnames (cdr checksum) *load-pathname*)))
	   (format t "Incorrect checksum on file: ~a" (cdr checksum))
	   (return nil)))))
  (cond
   ((not files-ok)
    (error "The extracted files failed the checksum test.  ~
Please download and/or extract again."))
   (t
    (format t "Checksums on files are OK...~%"))))

;; We have validated all the files are there...

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; check for needed patches

(eval-when (compile eval load) (require :build))

(defvar *required-patches* '(#+macosx ("build" . 3)))

(dolist (required-patch *required-patches*)
  (let ((existing-patch (assoc (car required-patch)
			    (cdr (assoc :system sys::*patches*))
			    :test #'string=)))
    (when (null existing-patch)
      (error "Could not find patch info for the ~s module."
	     (car required-patch)))
    (when (not (>= (second existing-patch) (cdr required-patch)))
      ;; The build patch number is NOT >= to the one we require
      (error "~
Expected version ~d of ~s but have version ~d.  Please update ~
your lisp with update.~a and then do the CG/JS installation again."
	     (cdr required-patch) (car required-patch)
	     (second existing-patch)
	     #+mswindows "exe"
	     #-mswindows "sh"))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; copy files

(defun copy (source dest)
  (cond
   ((file-directory-p source)
    (format t "Copy directory ~a~%" (enough-namestring source))
    (copy-directory source dest :quiet t))
   (t
    (format t "Copy file ~a~%" (enough-namestring source))
    (sys:copy-file source dest :verbose nil))))

(defvar *sys* (translate-logical-pathname (pathname "sys:")))

(cond
 ((excl::pathname-equalp (truename *sys*)
			 (path-pathname *load-truename*))
  (format t "Files are already in place, no copying requiring.~%"))
 (t ;; copy files, replacing them with new ones if they already exist
  (let (dest)
    (dolist (from *extra-files*)
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      (when (probe-file dest) (delete-file dest))
      (copy from dest))
    
    (dolist (from (list *cg2js-directory* *idejs-directory*))
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      ;; now make it a directory pathname
      (setq dest (pathname-as-directory dest))

      (when (probe-file dest) (delete-directory-and-files dest))
      (copy (pathname-as-directory from) dest)))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; update develenv.cl to load CG/JS before the original version of CG

(defvar *setup-cgjs-loading* "sys:setup-cgjs-loading.cl")

;; Because users might use an ANSI lisp to install, have to make sure
;; the resulting file can be loaded into an ANSI and Modern lisp.
(with-open-file (s *setup-cgjs-loading* :direction :output
		 :if-exists :supersede)
  (format s "~
;; This file was created by install-cgjs.cl.
;; Please do not edit it.

")
  (dolist (dir (list "code.cg2js" "code.idejs"))
    (format s "
(let ((path (make-pathname
             :host \"sys\"
             :directory '(:relative \"~a\")
	     :type \"fasl\"
	     )))
  (format t \";; *require-search-list*: adding: ~~s~%\" path)
  (push path sys:*require-search-list*))

"
	    dir)))

(defvar *develenv.cl* (pathname "sys:develenv.cl"))

(when (not (probe-file *develenv.cl*))
  (setf (file-contents *develenv.cl*)
    ";; This file is loaded before images are rebuilt with update.sh/update.exe

"))

(let ((contents (file-contents *develenv.cl*)))
  (cond
   ((match-re
     (format nil "^#\\|\\|# \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl already correctly loads CG/JS.~%"))
   ((match-re
     (format nil "^#\\+ignore \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl: CG/JS disabled; will enable.~%")
    (setf (file-contents *develenv.cl*)
      (replace-re  contents
		  (format nil "^#\\+ignore *\\(load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  (format nil "#||# (load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  :multiple-lines t
		  :count 1)))
   (t
    (format t "develenv.cl: adding CG/JS initialization.~%")
    (setf (file-contents *develenv.cl*)
      (concatenate 'simple-string
	contents
	(format nil "~

;; To disable the use of CG/JS and go back to the originally released
;; (and possibly patched) version of CG/IDE, change the #||# to
;; #+ignore on the following line and the rebuild images by running
;; update.sh or update.exe, depending on your platform:
#||# (load \"sys:~a\")~%~%"
		(file-namestring *setup-cgjs-loading*)))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; success!

(let ((msg (format nil "~
CG/JS installation successful!

NOTE: to use CG/JS you must rebuild images by running 'update.~a'
      in the Allegro CL directory (~a)."
		   #+mswindows "exe"
		   #-mswindows "sh"
		   *sys*)))
  #-mswindows (format t "~&~%~a~%" msg)
  #+mswindows
  (let ((noninteractive (sys:getenv "ACL_NONINTERACTIVE")))
    (when (or (null noninteractive)
	      (string= "" noninteractive))
      (excl::internal-message-box msg "CG/JS Installation" :type :ok))))
